/**
 * Copyright (c) 2017-present, Facebook, Inc. All rights reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 *
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _BaseWorkerPool;

function _load_BaseWorkerPool() {
  return (_BaseWorkerPool = _interopRequireDefault(
    require('./base/BaseWorkerPool')
  ));
}

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}

const canUseWorkerThreads = () => {
  try {
    // $FlowFixMe: Flow doesn't know about experimental APIs
    require('worker_threads');
    return true;
  } catch (_) {
    return false;
  }
};

class WorkerPool extends (_BaseWorkerPool || _load_BaseWorkerPool()).default {
  send(workerId, request, onStart, onEnd) {
    this.getWorkerById(workerId).send(request, onStart, onEnd);
  }

  createWorker(workerOptions) {
    let Worker;
    if (canUseWorkerThreads()) {
      Worker = require('./workers/NodeThreadsWorker').default;
    } else {
      Worker = require('./workers/ChildProcessWorker').default;
    }

    return new Worker(workerOptions);
  }
}

exports.default = WorkerPool;
