/**
 * Copyright (c) 2017-present, Facebook, Inc. All rights reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 *
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _mergeStream;

function _load_mergeStream() {
  return (_mergeStream = _interopRequireDefault(require('merge-stream')));
}

var _path;

function _load_path() {
  return (_path = _interopRequireDefault(require('path')));
}

var _types;

function _load_types() {
  return (_types = require('../types'));
}

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}

/* istanbul ignore next */
const emptyMethod = () => {};

class BaseWorkerPool {
  constructor(workerPath, options) {
    this._options = options;
    this._workers = new Array(options.numWorkers);

    if (!(_path || _load_path()).default.isAbsolute(workerPath)) {
      workerPath = require.resolve(workerPath);
    }

    const stdout = (0, (_mergeStream || _load_mergeStream()).default)();
    const stderr = (0, (_mergeStream || _load_mergeStream()).default)();

    const forkOptions = options.forkOptions,
      maxRetries = options.maxRetries,
      setupArgs = options.setupArgs;

    for (let i = 0; i < options.numWorkers; i++) {
      const workerOptions = {
        forkOptions: forkOptions,
        maxRetries: maxRetries,
        setupArgs: setupArgs,
        workerId: i,
        workerPath: workerPath
      };

      const worker = this.createWorker(workerOptions);
      const workerStdout = worker.getStdout();
      const workerStderr = worker.getStderr();

      if (workerStdout) {
        stdout.add(workerStdout);
      }

      if (workerStderr) {
        stderr.add(workerStderr);
      }

      this._workers[i] = worker;
    }

    this._stdout = stdout;
    this._stderr = stderr;
  }

  getStderr() {
    return this._stderr;
  }

  getStdout() {
    return this._stdout;
  }

  getWorkers() {
    return this._workers;
  }

  getWorkerById(workerId) {
    return this._workers[workerId];
  }

  createWorker(workerOptions) {
    throw Error('Missing method createWorker in WorkerPool');
  }

  end() {
    // We do not cache the request object here. If so, it would only be only
    // processed by one of the workers, and we want them all to close.
    for (let i = 0; i < this._workers.length; i++) {
      this._workers[i].send(
        [(_types || _load_types()).CHILD_MESSAGE_END, false],
        emptyMethod,
        emptyMethod
      );
    }
  }
}
exports.default = BaseWorkerPool;
